import plugin, json, argparse, shlex, traceback, sys

def get_parser(bot,channel,nick,subcmd):
	parser = argparse.ArgumentParser(prog=subcmd,add_help=False)
	parser.add_argument("-h","--help")
	def error(msg,*args,**kwargs):
		bot.say(channel,"{}: {}".format(nick,parser.format_usage().strip("\r\n")))
	parser.error = error
	return parser

MINECRAFT_CHANNELS = ["#minecraft","#minecraft-rand"]

# Parse the tcirc bot's messages
# Example usage:
# if nick=="tildecraft_mc_bot":
#     ok, info = parse_tcirc(msg)
#     if not ok: return # not a chat message
#     print(info)
import re
MC_MSG = re.compile(r"\[([A-Za-z\-_]+)\]([~A-Za-z0-9]+): (.+)")
IRC_MSG = re.compile(r"(IRC) > ([~A-Za-z0-9]+)@tilde.town: (.+)")
def parse_tcirc(line):
	m = MC_MSG.match(line)
	if not m: # not an in-game message
		m = IRC_MSG.match(line)
		if not m: return False, dict() # not a message we can parse info out of
	world, user, msg = m.groups()
	is_nick = user.startswith("~")
	info = dict(in_game=(world!="IRC"),user=user,is_nick=is_nick,msg=msg)
	return True, info

@plugin.group("minecraft")
def minecraft(bot,channel,nick,subcmd,*args):
	return False

@plugin.listener("minecraft_commands")
def mc_command_listener(bot,channel,nick,msg):
	if channel not in MINECRAFT_CHANNELS: return
	if nick=="tildecraft_mc_bot":
		ok, info = parse_tcirc(msg)
#		bot.say("khuxkm",(json.dumps(info) if ok else "doesn't parse"))
		if not ok: return
		if not info["msg"].startswith("."): return
		parts = info["msg"][1:].split()
		minecraft(bot,channel,(info["user"].lstrip("~")),*parts)

#@minecraft.command("test")
#def test(bot,channel,nick,subcmd,*msg):
#	bot.say(channel,"Nick is {} and args are {!r}".format(nick,msg))

@minecraft.command("calc")
def minecraft_calc(bot,channel,nick,subcmd,*args):
	try:
#		bot.say("khuxkm",json.dumps(dict(channel=channel,nick=nick,subcmd=subcmd,args=args)))
		args = shlex.split(" ".join(args))
		parser = get_parser(bot,channel,nick,subcmd)
		parser.add_argument("-s","--stack-size",type=int,default=64)
		parser.add_argument("stacks",type=int)
		parser.add_argument("items_left_over",type=int)
		args = parser.parse_args(args)
#		bot.say("khuxkm","{!r}".format(args))
		if args.items_left_over is None: return
		bot.say(channel,nick+": {} item(s)".format((args.stacks*args.stack_size)+args.items_left_over))
	except Exception as e:
		etype, e, tb = sys.exc_info()
		bot.say(channel,"ACCESS VIOLATION: "+";".join([x.strip() for x in traceback.format_exception_only(etype,e)]))
		pass
